package	jp.co.sds_corp.transfer.layout;

import java.util.List;
import java.util.Vector;

/**
* フォーマット文字列を解析します。
*/
public abstract class FormatParser
{
	static final int MODE_LITERAL=0;
	static final int MODE_ESCAPE=1;
	static final char ESCAPE='%';
	static final char BS='\\';

	String	fmt;
	int		nLen;
	int		nPos;
	int		nMode = MODE_LITERAL;
	Vector<IConverter>	vec = new Vector<IConverter>();
	StringBuffer buf = new StringBuffer();

	/**
	* 指定されたフォーマット文字列でFormatParserを作成します。
	* @param	fmt		フォーマット文字列
	*/
	public FormatParser( String fmt ) {
		this.fmt = fmt;
		nLen = fmt.length();
	}

	/**
	* フォーマット文字列を解析し、{@link IConverter}が格納された
	* {@link List}を返します。
	* @return	文字列の編集を行なうIConverter
	*/
	public List<IConverter> parse() {
		nPos = 0;
		while ( nPos < nLen ) {
			char c = fmt.charAt( nPos++ );
			switch ( nMode ) {
			case MODE_LITERAL:
				if ( nPos == nLen ) {
					buf.append( c );
					continue;
				}
				if ( c == ESCAPE ) {
					switch( fmt.charAt( nPos ) ) {
					case '%':
						buf.append( c );
						nPos++;
						break;
					default:
						if ( buf.length() > 0 ) {
							vec.add( new LiteralConverter( buf.toString() ));
						}
						buf.setLength( 0 );
						buf.append( c );
						nMode = MODE_ESCAPE;
					}
				} else if ( c == BS ) {
					switch( fmt.charAt( nPos ) ) {
					case '\\':
						buf.append( c );
						nPos++;
						break;
					case 'n':
						buf.append( '\n' );
						nPos++;
						break;
					case 't':
						buf.append( '\t' );
						nPos++;
						break;
					default:
						if ( buf.length() > 0 ) {
							vec.add( new LiteralConverter( buf.toString() ));
						}
						buf.setLength( 0 );
						buf.append( c );
					}
				} else {
					buf.append( c );
				}
				break;
			case MODE_ESCAPE:
				buf.append( c );
				IConverter conv = createConverter( c );
				if ( conv == null ) {
					conv = new LiteralConverter( buf.toString() );
				}
				vec.add( conv );
				nMode = MODE_LITERAL;
				buf.setLength( 0 );
				break;
			}
		}
		if ( buf.length() > 0 ) {
			vec.add( new LiteralConverter( buf.toString() ) );
		}
		return vec;
	}

	/**
	* 指定されたコンバータを生成します。
	* @param	c		コンバータを指定する文字
	* @return	コンバータ
	*/
	protected abstract IConverter createConverter( char c );

	/**
	* オプションのフォーマット文字列を返します。
	* @return	フォーマット文字列
	*/
	protected String getOption() {
		if (( nPos < nLen ) && ( fmt.charAt( nPos ) == '{' )) {
			int end = fmt.indexOf( '}', nPos );
			if ( end > nPos ) {
				String r = fmt.substring( nPos + 1, end );
				nPos = end + 1;
				return r;
			}
		}
		return null;
	}

}
