package	jp.co.sds_corp.transfer.source;

import jp.co.sds_corp.transfer.IReceiveListener;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.transfer.output.EventAdapter;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* データをJMSで受信します。
各要素の詳細については、util.msgを参照してください。
<PRE>
&lt;Source name="Event" class="jp.co.sds_corp.transfer.source.EventSource"&gt;
	&lt;name&gt;NAME&lt;/name&gt;
&lt;/Source&gt;
</PRE>
*/
public class EventSource extends AbstractSource implements IReceiveListener
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String name;
	EventAdapter adapter;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		name = config.get( "name" );
		adapter = EventAdapter.getAdapter( name );
		if ( adapter == null ) {
			throw( new Exception( "EventAdapter not found.[" + name + "]" ));
		}
		adapter.addReceiveListener( this );
	}

	/**
	* このサービスを開始します。
	*/
	public void start() {
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		adapter.removeReceiveListener( this );
		logger.debug( this, "stopped." );
	}

	/**
	* データに何らかの変更があったときに呼び出されます。
	* @param	e		イベントソースおよびデータの変更内容を指定します。
	*/
	public void received( ReceiveEvent e ) throws Exception {
		if ( isDebug() ) {
			logger.debug( this, e.toString() );
		}
		fireReceive( e );
	}

	/**
	* 状態に何らかの変更があったときに呼び出されます。
	* @param    e       イベントソースおよび状態の変更内容を指定します。
	*/
	public void statusChanged( StatusChangeEvent e ) throws Exception {
		if ( isDebug() ) {
			logger.debug( this, e.toString() );
		}
		fireStatusChange( e );
	}

	/**
	* この取得元の文字列表現を返します。
	* @return	この取得元の文字列表現
	*/
	public String toString() {
		String ret = super.toString();
		ret += ":name=[" + name + "]";
		return ret;
	}

}
